/*
 *      quickvpn.c
 *      
 *      Copyright 2008 Fabio DM <pobfdm@gmail.com>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <gtk/gtk.h>
#include <signal.h>
#include <gio/gio.h>



#include <dirent.h>
#include "message.c"


#define TERM "xterm -bg white -fg black -e "
#define VERSION "QuickVPN 0.3"
#define LICENSE "This program is free software; you can redistribute it and/or modify \n \
it under the terms of the GNU General Public License as published by \n \
the Free Software Foundation; either version 2 of the License, or \n \
(at your option) any later version."
enum {
        TARGET_INT32,
        TARGET_STRING,
		TARGET_URL,
        TARGET_ROOTWIN
	};	   
GtkWidget *MainWindow;
GtkWidget *cmbStartStop;
GtkWidget *MainFixed;
GtkWidget *cmbEsci;
GtkWidget *lblStatus;
GtkWidget *Tray;
GtkWidget *VpnsGrid;
GtkListStore  *VpnsGridStore; //data container for Vpns grid
GtkWidget *Locked;
GdkPixbuf *LockedRed, *LockedGray, *LockedGreen;

gchar *FileApp;
gchar *DirApp;
gchar *VpnsDir; 
gchar *CurrentVpnDir;

const gchar *PIDfile="/tmp/quick_vpn.pid";
const gchar *StatusFile="/tmp/quick_vpn.status";
const gchar *ErrFile="/tmp/quick_vpn.err";//non usato
const gchar *LogFile="/tmp/quick_vpn.log";//non usato

//Prototipi
int controlloreVpn();
static GtkWidget *InitAboutWindow();
static void VpnDelete(GtkWidget *widget, gpointer data);
static void
drag_data_received
(GtkWidget *widget, GdkDragContext *context, gint x, gint y,
        GtkSelectionData *selection_data, guint target_type, guint time,
        gpointer data);
static void VpnAdd(GtkWidget *widget, gpointer data);
static void CloseMainWindow( GtkWidget *widget, gpointer   data );
void UpdateVpnGrid(GtkWidget *view);

static GtkItemFactoryEntry menu_items[] = {
  { "/_VPNs",         NULL,         NULL,           0, "<LastBranch>" },
  { "/_VPNs/_Add",     NULL,         VpnAdd,           0, "<StockItem>",GTK_STOCK_ADD},
  { "/_VPNs/_Delete",     NULL,      VpnDelete,           0, "<StockItem>",GTK_STOCK_DELETE},
  { "/_VPNs/Exit",     NULL,      CloseMainWindow,           0, "<StockItem>",GTK_STOCK_QUIT},
  { "/_Help",         NULL,         NULL,           0, "<LastBranch>" },
  { "/_Help/About",   NULL , InitAboutWindow, 0, "<StockItem>",GTK_STOCK_ABOUT},
};


static void VpnAdd(GtkWidget *widget, gpointer data)
{
	
	GtkWidget *dialog;
	GtkFileFilter* filter;
	
	
	dialog = gtk_file_chooser_dialog_new ("Select a folder",
						  GTK_WINDOW(MainWindow),
						  GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
						  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
						  GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
						  NULL);
	//Filter for directory
	filter=gtk_file_filter_new();
	gtk_file_filter_add_mime_type(GTK_FILE_FILTER(filter),"inode/directory");
	gtk_file_filter_set_name(GTK_FILE_FILTER(filter),"Directory");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog),filter);
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
	  {
		gchar *filename, *basename, *destlink;
		
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		basename=g_filename_display_basename(filename);
		destlink=g_build_filename(VpnsDir,basename,NULL);
		
		GFile *LinkFile=g_file_new_for_path(destlink); 
		
		if(g_file_make_symbolic_link(LinkFile, filename, NULL,NULL )==TRUE)
		{
			UpdateVpnGrid(VpnsGrid);
		}else{
			err_message(GTK_WINDOW(MainWindow), "Sorry, I can't make a link for you Vpn.");
		}
		g_free (filename);
	  }
	gtk_widget_destroy (dialog);
}

static void VpnDelete(GtkWidget *widget, gpointer data)
{
	if (info_YesNo(GTK_WINDOW(MainWindow),"Do you want really delete this Vpn's file?"  )==GTK_RESPONSE_YES)
	{
		 if (g_file_test (CurrentVpnDir, G_FILE_TEST_IS_SYMLINK)){
		 	if (g_unlink(CurrentVpnDir)!=0) 
					err_message(GTK_WINDOW(MainWindow),"Error on delete file!");
		 }
		 if (g_file_test (CurrentVpnDir,G_FILE_TEST_IS_DIR)){
			 if (g_rmdir(CurrentVpnDir)!=0)
			 	err_message(GTK_WINDOW(MainWindow),"Error on delete folder!");
		}
		UpdateVpnGrid(VpnsGrid);
	}
}


static GtkWidget *get_menubar_menu( GtkWidget  *window )
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;

  /* Make an accelerator group (shortcut keys) */
  accel_group = gtk_accel_group_new ();

  /* Make an ItemFactory (that makes a menubar) */
  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
                                       accel_group);

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
	static gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  /* Finally, return the actual menu bar created by the item factory. */
  return gtk_item_factory_get_widget (item_factory, "<main>");
}

static GtkWidget *InitAboutWindow()
{
	const gchar *authors[]={
		("Fabio Di Matteo - pobfdm@gmail.com"), ("And you? - your@email.com")
		,NULL};
	
	const gchar *artists[]={
		("Alexandre Moore http://www.gnome-look.org/content/show.php?content=28352"), ("And you? - your@email.com")
		,NULL};
	
	
	GtkWidget *AboutWindow= gtk_about_dialog_new();
	gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(AboutWindow),VERSION);
	gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(AboutWindow),authors);
	gtk_about_dialog_set_artists(GTK_ABOUT_DIALOG(AboutWindow),artists);
	gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(AboutWindow),LICENSE);
	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(AboutWindow),("http://www.freemedialab.org"));
	gtk_dialog_run(GTK_DIALOG(AboutWindow));
	gtk_widget_destroy(GTK_WIDGET(AboutWindow));
	
	
}


/*int TimeOutRead(){ //not used
		gchar *TimeOutfile=g_build_filename(DirApp,"TIMEOUT",NULL);
		if (g_file_test (TimeOutfile, G_FILE_TEST_EXISTS )){
			gchar TimeOut[6];
			FILE *f=fopen(TimeOutfile, "r");
			fgets(TimeOut,6,f);
			fclose(f);
			return atoi(TimeOut);
		}else{
			return 5;
		}
}*/

int PidRead(){
		char pid[6];
		if (g_file_test (PIDfile, G_FILE_TEST_EXISTS )){
			FILE *f=fopen(PIDfile, "r");
			fgets(pid,6,f);
			fclose(f);
		}
		
		return atoi(pid);
}

/*char* PassphraseRead(){ //not used
		gchar *FilePassphrase=g_build_filename(DirApp,"configurazione/passphrase",NULL);
		gchar Passphrase[80];
		FILE *f=fopen(FilePassphrase, "r");
		fgets(Passphrase,80,f);
		fclose(f);
		return Passphrase;
}*/

void PuliziaFile(){
	if (g_file_test (PIDfile, G_FILE_TEST_EXISTS )) g_unlink(PIDfile);
    if (g_file_test (ErrFile, G_FILE_TEST_EXISTS )) g_unlink(ErrFile);	
    if (g_file_test (StatusFile, G_FILE_TEST_EXISTS )) g_unlink(StatusFile);
    if (g_file_test (LogFile, G_FILE_TEST_EXISTS )) g_unlink(LogFile);
}


	
gboolean VpnIsStart(){
		char buff;
		gboolean res=FALSE;
		if (g_file_test (StatusFile, G_FILE_TEST_EXISTS )==TRUE){
			FILE *f=fopen(StatusFile, "r");
			buff=fgetc(f);
			fclose(f);
			if (buff=='1'){
				res=TRUE;
			}else{
				res=FALSE;
			}
		}else{
			res=FALSE;
		}	
			
		return res;	
}


static void CloseMainWindow( GtkWidget *widget, gpointer   data )
{
    //Chiude la VPN
	int pid= PidRead();
	if (pid!=0){
		if (kill((pid_t) pid, SIGTERM)<0){
			//Non e' stato possibile chiudere la Vpn
			info_message(GTK_WINDOW(MainWindow),"Non e\' stato possibile chiudere la connessione!");
		}
	}
    PuliziaFile();
    gtk_main_quit ();
}


static void StartStop(GtkWidget *widget, gpointer data)
{
	//If present use OpenVpn binary included on quickvpn
	gchar *OpenVpnBin=g_build_filename(DirApp,"openvpn",NULL);
	if (g_file_test (OpenVpnBin, G_FILE_TEST_EXISTS )==FALSE) 
									OpenVpnBin=g_strdup("openvpn");
	
	gchar *CmdStart=g_strconcat(TERM ,"\"",OpenVpnBin, " --verb 0 --log ",LogFile,
	" --daemon --config "	,CurrentVpnDir,"/openvpn.conf --writepid ",
	PIDfile, " --cd ", CurrentVpnDir, " --up 'echo 1 > ",StatusFile,
	"' --down 'echo 0 > ",StatusFile , " ' \" ", NULL);
	
	
	
	//debug
	g_print("Pid file -> %s\n",PIDfile);
	g_print("Open VPN -> %s\n",OpenVpnBin);
	g_print("CurrentVpnDir -> %s\n",CurrentVpnDir);
	g_print("Comando -> %s\n",CmdStart);
	
	//Hide main window...
	//gtk_widget_hide(GTK_WIDGET(MainWindow));
	
	
	
	//Check if openvpn is started
	if(VpnIsStart()==FALSE){
		system(CmdStart); 
	}else{
		//Tenta di chiudere la VPN
		int pid= PidRead();
		if (kill(pid, SIGTERM)<0){
				//Non e' stato possibile chiudere la Vpn
				err_message(GTK_WINDOW(MainWindow),"Non e\' stato possibile chiudere la connessione!");
			}else{
				//E' stato possibile chiudere la vpn
				if (g_file_test (PIDfile, G_FILE_TEST_EXISTS )) g_unlink(PIDfile);
			}
	}
	
}

static void TrayClick(GtkWidget *widget, gpointer data)
{
	if ( gtk_window_is_active(GTK_WINDOW(MainWindow))==TRUE ){
		gtk_widget_hide(GTK_WIDGET(MainWindow));
	}else{
		gtk_widget_show_all(GTK_WIDGET(MainWindow));
		gtk_window_deiconify(GTK_WINDOW(MainWindow));
	}
}

static void HideMainWindow(GtkWidget *widget, gpointer data)
{
	gtk_widget_hide(GTK_WIDGET(MainWindow));
}


int controlloreVpn()
{
		if (VpnIsStart()==TRUE){
			gtk_button_set_label(GTK_BUTTON(cmbStartStop),"gtk-disconnect");
			gtk_status_icon_set_from_pixbuf(GTK_STATUS_ICON(Tray),GDK_PIXBUF(LockedGreen));
			gtk_status_icon_set_tooltip(GTK_STATUS_ICON(Tray), "Vpn connessa.");
			gtk_image_set_from_pixbuf(GTK_IMAGE(Locked),GDK_PIXBUF(LockedGreen));
			gtk_widget_set_sensitive(GTK_WIDGET(VpnsGrid),FALSE);
		}else{
			gtk_button_set_label(GTK_BUTTON(cmbStartStop),"gtk-connect");
			gtk_status_icon_set_from_pixbuf(GTK_STATUS_ICON(Tray),GDK_PIXBUF(LockedRed));		
			gtk_status_icon_set_tooltip(GTK_STATUS_ICON(Tray), "Vpn non connessa.");
			gtk_image_set_from_pixbuf(GTK_IMAGE(Locked),GDK_PIXBUF(LockedRed));
			gtk_widget_set_sensitive(GTK_WIDGET(VpnsGrid),TRUE);
			
		}
		
	//Debug
	//g_print("Ciao sono il thread :) \n ");
	return TRUE ;
}


void UpdateVpnGrid(GtkWidget *view)
{
	enum { COL_IMG  , COL_VPN, NUM_COLS } ;
	/*Container not visible*/
    GtkTreeIter    iter;
  	GdkPixbuf     *pixbuf;
	pixbuf = gdk_pixbuf_new_from_file(g_build_filename(DirApp,"/img/locked_32.png",NULL), NULL);

	/*Visual Container*/
	GtkCellRenderer     *renderer; //la cella
	
	gtk_list_store_clear(GTK_LIST_STORE(VpnsGridStore));
	
	/*Scan Vpn dir*/
	struct dirent **namelist; 
	int n, i=1; 
	 
	n = scandir(VpnsDir, &namelist, 0, alphasort); 
	if (n < 0){ 
		perror("No vpn found!\n"); 
	}else{
		while(n--){
			if (strcmp(namelist[n]->d_name,".")!=0 && strcmp(namelist[n]->d_name,"..")!=0){
				/* Add a row */
				gtk_list_store_append (VpnsGridStore, &iter);
				gtk_list_store_set (VpnsGridStore, &iter,COL_IMG, pixbuf, COL_VPN, namelist[n]->d_name,-1);
			}		
		}
	}
	
  GtkTreeModel  *ModelUpdate=GTK_TREE_MODEL (VpnsGridStore); 
  gtk_tree_view_set_model (GTK_TREE_VIEW (view), ModelUpdate);
}

GtkWidget *InitVpnGrid()
{
	enum { COL_IMG  , COL_VPN, NUM_COLS } ;
	
	/*Container not visible*/
	
    GtkTreeIter    iter;  //segnaposto griglia
  	GdkPixbuf     *pixbuf;
	pixbuf = gdk_pixbuf_new_from_file(g_build_filename(DirApp,"/img/locked_32.png",NULL), NULL);

	/*Visual Container*/
	GtkCellRenderer     *renderer; //la cella
  	GtkWidget           *view;
	view = gtk_tree_view_new();
	
	
	//Creo il contenitore dei dati, specificando il tipo per ogni colonna
  	VpnsGridStore = gtk_list_store_new (NUM_COLS, GDK_TYPE_PIXBUF, G_TYPE_STRING);
	
	/*Scan Vpn dir*/
	struct dirent **namelist; 
	int n, i=1; 
	 
	n = scandir(VpnsDir, &namelist, 0, alphasort); 
	if (n < 0){ 
		perror("No vpn found!\n"); 
	}else{
		while(n--){
			if (strcmp(namelist[n]->d_name,".")!=0 && strcmp(namelist[n]->d_name,"..")!=0){
				/* Aggiungiamo una riga */
				gtk_list_store_append (VpnsGridStore, &iter);
				gtk_list_store_set (VpnsGridStore, &iter,COL_IMG, pixbuf, COL_VPN, namelist[n]->d_name,-1);
			}		
		}
	}
	
	
   /* --- C1 --- */
  renderer = gtk_cell_renderer_pixbuf_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
                                               -1,      
                                               "",  
                                               renderer,
                                               "pixbuf", COL_IMG,
                                               NULL);
	/* --- C2 --- */

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
                                               -1,      
                                               "VPNs",  
                                               renderer,
                                               "text", COL_VPN,
                                               NULL);
											   
  GtkTreeModel  *model=GTK_TREE_MODEL (VpnsGridStore);
  gtk_tree_view_set_model (GTK_TREE_VIEW (view), model);
  g_object_unref (model);   
  
  //For Drad and Drop
  //http://live.gnome.org/GnomeLove/DragNDropTutorial
  
  static GtkTargetEntry target_list[] = {
        //{ "INTEGER",    0, TARGET_INT32 },
        //{ "STRING",     0, TARGET_STRING },
        //{ "text/plain", 0, TARGET_STRING },
		{ "text/uri-list", 0, TARGET_URL },
        //{ "application/x-rootwindow-drop", 0, TARGET_ROOTWIN }
	};
 static guint n_targets = G_N_ELEMENTS (target_list);
 gtk_drag_dest_set(GTK_WIDGET(view), GTK_DEST_DEFAULT_ALL, target_list,n_targets,GDK_ACTION_COPY);

  return GTK_WIDGET (view);

}

gboolean GridVpnsCursorChanged (GtkTreeView *tree_view, gpointer user_data) 
{
	GtkTreeIter   iter;
  	GtkTreeModel *model;
	GtkTreeSelection *selection;
	gchar *VPN, *labelVPN;
	
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	model = gtk_tree_view_get_model(tree_view);
  	
	if (gtk_tree_selection_get_selected(selection, &model, &iter))
  	{
		gtk_tree_model_get(model, &iter, 1, &VPN, -1);
		//Only for label
		labelVPN = g_markup_printf_escaped("<b>%s</b>", VPN);
    	gtk_label_set_markup(GTK_LABEL(lblStatus),labelVPN);
		
		g_print ("CurrentVpn->%s\n", VPN);
    	CurrentVpnDir=g_build_filename(VpnsDir,VPN,NULL);
		g_free(VPN);
  	}
	gtk_widget_set_sensitive(GTK_WIDGET(cmbStartStop),TRUE);
	
}


void
onGridVpnsRowChanged (GtkTreeView *view, GtkTreePath *path,
                        GtkTreeViewColumn *col, gpointer userdata)
{
 	GtkTreeIter   iter;
  	GtkTreeModel *model;
	gchar *VPN, *labelVPN;
	
  	model = gtk_tree_view_get_model(view);

  if (gtk_tree_model_get_iter(model, &iter, path))
  {
    	gtk_tree_model_get(model, &iter, 1, &VPN, -1);
		
		//Only for label
		labelVPN = g_markup_printf_escaped("<b>%s</b>", VPN);
    	gtk_label_set_markup(GTK_LABEL(lblStatus),labelVPN);
		
		g_print ("CurrentVpn->%s\n", VPN);
    	CurrentVpnDir=g_build_filename(VpnsDir,VPN,NULL);
		g_free(VPN);
		StartStop(NULL,NULL);
  }
	gtk_widget_set_sensitive(GTK_WIDGET(cmbStartStop),TRUE);
  
}




void InitMainWindow()
{
if (getuid()==0){	
	//Finestra principale
	MainWindow= gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (MainWindow), VERSION );
	gtk_window_set_resizable(GTK_WINDOW(MainWindow),1);
	gtk_window_set_position(GTK_WINDOW(MainWindow),GTK_WIN_POS_MOUSE);
	//gtk_window_set_default_size(GTK_WINDOW(MainWindow), 200, 450);
	gtk_window_set_decorated(GTK_WINDOW(MainWindow),TRUE);
	gtk_window_set_resizable(GTK_WINDOW(MainWindow),FALSE);
	gtk_window_set_icon_from_file(GTK_WINDOW(MainWindow),g_build_filename(DirApp,"/img/locked_gray.png",NULL),NULL);
	
	//Vpns Grid
	VpnsGrid =InitVpnGrid();
	GtkWidget *GridScrollbar=gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_hadjustment(GTK_SCROLLED_WINDOW(GridScrollbar),NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(GridScrollbar),GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(GridScrollbar), GTK_WIDGET(VpnsGrid));
	gtk_widget_set_size_request(GTK_WIDGET(GridScrollbar),190,200);
	
	
	//Lockeds
	LockedGray=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"/img/locked_gray.png",NULL), NULL);
	LockedGreen=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"/img/locked_green.png",NULL), NULL);
	LockedRed=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"/img/locked_red.png",NULL), NULL);
	
	
	//Big Image locked
	Locked=gtk_image_new_from_pixbuf(GDK_PIXBUF(LockedGray));
	
	
	//Tray icon
	Tray=gtk_status_icon_new_from_pixbuf(GDK_PIXBUF(LockedGray));
	
	
	//Pulsanti
	cmbStartStop=gtk_button_new_from_stock("gtk-connect");
	//gtk_widget_set_sensitive(GTK_WIDGET(cmbStartStop),FALSE);
	lblStatus=gtk_label_new("<b>Select a vpn...</b>");
	 gtk_label_set_justify(GTK_LABEL(lblStatus),GTK_JUSTIFY_CENTER);
	gtk_label_set_use_markup(GTK_LABEL(lblStatus),1);
	cmbEsci=gtk_button_new_from_stock("gtk-quit");
	
	//Creiamo il contenitore per la finestra MainWindow
	MainFixed=gtk_fixed_new();
	gtk_widget_set_size_request(GTK_WIDGET(MainFixed),200,450);
	gtk_container_add (GTK_CONTAINER (MainWindow), MainFixed);
  	
	//Menubar 
	GtkWidget *MenuBar=gtk_menu_bar_new();
	MenuBar = get_menubar_menu (MainWindow);
	
	
  	//Aggiungiamo i widgets al contenitore
	gtk_fixed_put(GTK_FIXED(MainFixed),MenuBar,1,1);
  	gtk_fixed_put(GTK_FIXED(MainFixed),cmbStartStop,55,340);
  	gtk_fixed_put(GTK_FIXED(MainFixed),cmbEsci,70,400);
  	gtk_fixed_put(GTK_FIXED(MainFixed),lblStatus,95,270);
  	gtk_fixed_put(GTK_FIXED(MainFixed),GridScrollbar,5,35);
	gtk_fixed_put(GTK_FIXED(MainFixed),Locked, 30,245);
  	
	//Mostriamo la finestra principale ed il suo contenuto
  	//gtk_widget_show_all(GTK_WIDGET(MainWindow));
	
	
	//Connettiamo le callbacks
	g_signal_connect (G_OBJECT (MainWindow), "delete-event",
		      G_CALLBACK (HideMainWindow), NULL);
	g_signal_connect (G_OBJECT (cmbEsci), "clicked",
		      G_CALLBACK (CloseMainWindow), NULL);
	g_signal_connect (G_OBJECT (cmbStartStop), "clicked",
		      G_CALLBACK (StartStop), NULL);
	g_signal_connect (G_OBJECT (Tray), "activate",
		      G_CALLBACK (TrayClick), NULL);
	g_signal_connect (G_OBJECT (VpnsGrid), "row-activated",
		      G_CALLBACK (onGridVpnsRowChanged), NULL);	
	g_signal_connect (G_OBJECT (VpnsGrid), "cursor-changed",
		      G_CALLBACK (GridVpnsCursorChanged), NULL);		  
	g_signal_connect (G_OBJECT (VpnsGrid), "drag-data-received",
		      G_CALLBACK (drag_data_received), NULL);		  
	

}else{
	
	if (system(g_strconcat("gksu -u root ", FileApp, " || kdesu  ",FileApp, NULL))< 0){
		err_message(GTK_WINDOW(MainWindow),"Si devono avere i privilegi di root per eseguire QuickVPN.");
	}else{
		gtk_main_quit();
	}


}	
	
}	



static void
drag_data_received
(GtkWidget *widget, GdkDragContext *context, gint x, gint y,
        GtkSelectionData *selection_data, guint target_type, guint time,
        gpointer data)
{
	if((selection_data != NULL) && (selection_data-> length >= 0))
    {
         if(target_type==TARGET_URL)
		 {
		 		gchar *uri = (guchar*)selection_data->data;
				gchar *dirname=g_filename_from_uri(uri,NULL,NULL);
				dirname=g_filename_to_utf8(dirname,-1,NULL,NULL,NULL);
								
				gchar *dirbasename=g_filename_display_basename(dirname);
				gchar *destlink=g_build_filename(VpnsDir,dirbasename,NULL);
				
								
				g_print ("Dirname: %s\n", dirname);
				g_print ("Destlink: %s\n", destlink);
												
				gchar *cmd=g_strconcat("cp -Rf ",dirname," ",destlink,NULL);
				g_print("Comando ->%s\n",cmd);
				info_message(GTK_WINDOW(MainWindow),"Help me with this drag and drop!\n");
				
				/*GFile *LinkFile=g_file_new_for_path(destlink); 
				g_file_make_symbolic_link(LinkFile,
										dirname, NULL,NULL );*/
				//system(cmd);
				
				
				/*if (symlink(dirname,destlink)!=0)
				{
					err_message(GTK_WINDOW(MainWindow),"Error!");
				}*/
				
		 }
	}

}



int main(int argc, char* argv[])
{
	
	FileApp=argv[0];
	DirApp = g_path_get_dirname(argv[0]);
	
	if (argv[1]==NULL){
		VpnsDir=g_build_filename(DirApp,"VPNs/",NULL);
	}else {
		VpnsDir=g_build_filename(argv[1], NULL );
	}
	
	
	//Debug
	g_print("App Dir ->%s\n",DirApp);
	
	PuliziaFile();
		
	gtk_init (&argc, &argv);
	
	//Faccio partire un timer che controlla la connessione
	g_timeout_add(1000, controlloreVpn, NULL);
			
	//Disegna la finestra principale
	InitMainWindow();
	
	
	gtk_main ();
	return 0;
}
